using System;
using System.Collections;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Globalization;
using System.Text;
using System.Threading;
using System.Windows.Forms;

using Syncfusion.ComponentModel;
using Syncfusion.Drawing;
using Syncfusion.Styles;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;

namespace AccountTest
{
	[TypeConverter(typeof(ExpandableObjectConverter))]
	public class Account
	{
		private string _AccountId;
		private double _total;
		private AccountCollection.AddNewObjectListener _forwardEditableObject;

		#region Fields

		private AccountCollection _parentCollection;
		private AccountCollection _children = new AccountCollection();

		#endregion

		#region ctor

		/// <summary>
		/// Default ctor needed for Xml and CodeDom serialization of strong-typed collections and
		/// for Essential Grid to be able to detect properties of items in strong-typed collections.
		/// </summary>
		public Account()
		{
			_AccountId = "";
			_total = 0;
		}

		/// <summary>
		/// Specialized ctor
		/// </summary>
		/// <param name="ID"></param>
		public Account(string AccountId)
		{
			_AccountId = AccountId;
			_total = 0;
		}

		#endregion 

		#region Properties

		[Browsable(false)]
		public AccountCollection.AddNewObjectListener ForwardEditableObject
		{
			get { return _forwardEditableObject; }
			set { _forwardEditableObject = value; }
		}

		[Browsable(true)]
		public string AccountId
		{
			get { return _AccountId;	}
		}

		[Browsable(true)]
		public double Total
		{
			get { return _total; }
			set	
			{ 
				if(_total != value) 
				{ 
					_total = value; 
					int idx = ParentCollection.IndexOf(this);
					ParentCollection.RaiseListChanged(new ListChangedEventArgs(ListChangedType.ItemChanged, idx, idx)); 
				}			
			}
		}

		[Browsable(true)]
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Content)]
		public AccountCollection Children
		{
			get 
			{
				return _children;
			}
			set 
			{
				if (!Object.ReferenceEquals(Children, value))
				{
					_children = value;
				}
			}
		}

		public bool ShouldSerializeChildren()
		{
			return _children != null && _children.Count > 0;
		}

		[Browsable(false)] // do not show as field in grid
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
		public AccountCollection ParentCollection 
		{
			get { return _parentCollection;	}
			set { _parentCollection = value; }
		}

		public bool ShouldSerializeParentCollection()
		{
			return false;
		}

		#endregion

	}


	public class AccountCollection : IList, IBindingList, IListSource
	{
		ArrayList inner = new ArrayList();

		public AccountCollection()
		{
		}

		public override string ToString()
		{
			return GetType().Name.ToString() + "{ " + String.Format("Count {0}", Count) + " }";
		}

		public Account this[int index]
		{
			get
			{
				return (Account) inner[index];
			}
			set
			{
				if (!Object.ReferenceEquals(inner[index], value))
				{
					inner[index] = value;
					value.ParentCollection = this;
					RaiseListChanged(new ListChangedEventArgs(ListChangedType.ItemChanged, index, index));
				}
			}
		}

		public Account this[string accountId]
		{
			get 
			{
				foreach(Account acct in inner)
					if(acct.AccountId == accountId)
						return acct;

				return null;
			}
		}

		public bool Contains(Account value)
		{
			if (value == null)
				return false;

			return inner.Contains(value);
		}

		public int IndexOf(Account value)
		{
			return inner.IndexOf(value);
		}

		public void CopyTo(Account[] array, int index)
		{
			int count = Count;
			for (int n = 0; n < count; n++)
				array[index+n] = this[n];
		}

		public AccountCollection SyncRoot
		{
			get
			{
				throw new NotSupportedException();
			}
		}

		public AccountCollectionEnumerator GetEnumerator()
		{
			return new AccountCollectionEnumerator(this);
		}

		public void RefreshRow(int rowIdx)
		{
			RaiseListChanged(new ListChangedEventArgs(ListChangedType.ItemChanged, rowIdx, rowIdx));
		}
		
		public void Insert(int index, Account value)
		{
			inner.Insert(index, value);
			value.ParentCollection = this;
			RaiseListChanged(new ListChangedEventArgs(ListChangedType.ItemAdded, index, -1));
		}
	
		public void Remove(Account value)
		{
			int index = IndexOf(value);
			RemoveAt(index);
		}
		
		public int Add(Account value)
		{
			int index = inner.Add(value);
			value.ParentCollection = this;
			RaiseListChanged(new ListChangedEventArgs(ListChangedType.ItemAdded, index, -1));
			return index;
		}

		public void RemoveAt(int index)
		{
			inner.RemoveAt(index);
			RaiseListChanged(new ListChangedEventArgs(ListChangedType.ItemDeleted, index, -1));
		}

		public void Clear()
		{
			if (inner.Count > 0)
			{
				inner.Clear();
				RaiseListChanged(new ListChangedEventArgs(ListChangedType.Reset, -1, -1));
			}
		}

		public bool IsReadOnly
		{
			get
			{
				return false;
			}
		}

		public bool IsFixedSize
		{
			get
			{
				return false;
			}
		}

		public bool IsSynchronized
		{
			get
			{
				return false;
			}
		}

		public int Count
		{
			get
			{
				return inner.Count;
			}
		}


		public void NotifyItemPropertyChanged(Account item, string propertyName)
		{
			this.RaiseListChanged(new ListChangedEventArgs(ListChangedType.ItemChanged, IndexOf(item), -1));
		}

		#region IList Private Members

		object IList.this[int index]
		{
			get
			{
				return this[index];
			}
			set
			{
				this[index] = (Account) value;
			}
		}

		void IList.Insert(int index, object value)
		{
			Insert(index, (Account) value);
		}

		void IList.Remove(object value)
		{
			Remove((Account) value);
		}

		bool IList.Contains(object value)
		{
			return Contains((Account) value);
		}

		int IList.IndexOf(object value)
		{
			return IndexOf((Account) value);
		}

		int IList.Add(object value)
		{
			return Add((Account) value);
		}

		#endregion

		#region ICollection Private Members

		void ICollection.CopyTo(Array array, int index)
		{
			CopyTo((Account[]) array, index);
		}

		object ICollection.SyncRoot
		{
			get
			{
				return null;
			}
		}

		#endregion

		#region IEnumerable Private Members

		IEnumerator IEnumerable.GetEnumerator()
		{
			return GetEnumerator();
		}

		#endregion

		#region IBindingList Members

		// Implements IBindingList.
		public bool AllowEdit 
		{ 
			get { return true ; }
		}

		public bool AllowNew 
		{ 
			get { return true ; }
		}

		public bool AllowRemove 
		{ 
			get { return true ; }
		}

		public bool SupportsChangeNotification 
		{ 
			get { return true ; }
		}
        
		public bool SupportsSearching 
		{ 
			get { return false ; }
		}

		public bool SupportsSorting 
		{ 
			get { return false ; }
		}

		// Events.

		public event ListChangedEventHandler ListChanged;
		
		protected virtual void OnListChanged(ListChangedEventArgs e)
		{
			if (ListChanged != null)
				ListChanged(this, e);
		}
		
		public void RaiseListChanged(ListChangedEventArgs e)
		{
			OnListChanged(e);
		}

	
		// Methods.

		public Account AddNew()
		{
			Account c = new Account(this.Count.ToString());
			new AddNewObjectListener(this, c, Add(c));
			return c;
		}

		object IBindingList.AddNew() 
		{
			return AddNew();
		}

		public class AddNewObjectListener : IEditableObject	
		{
			AccountCollection Accounts;
			Account item;
			int index;

			public AddNewObjectListener(AccountCollection Accounts, Account item, int index)
			{
				this.Accounts = Accounts;
				this.item = item;
				this.index = index;
				item.ForwardEditableObject = this;
			}

			public void Detach()
			{
				item.ForwardEditableObject = null;
				Accounts = null;
				item = null;
			}

			#region IEditableObject Members

			public void EndEdit()
			{
				Accounts.RaiseListChanged(new ListChangedEventArgs(ListChangedType.ItemChanged, index, index));
				Detach();
			}

			public void CancelEdit()
			{
				// When a AddNew operation is canceld by the user the object needs to be removed again from
				// the list.
				Accounts.RemoveAt(index);
				Detach();
			}

			public void BeginEdit()
			{
			}

			#endregion
		}

		// Unsupported properties.
		bool IBindingList.IsSorted 
		{ 
			get { throw new NotSupportedException(); }
		}

		ListSortDirection IBindingList.SortDirection 
		{ 
			get { throw new NotSupportedException(); }
		}


		PropertyDescriptor IBindingList.SortProperty 
		{ 
			get { throw new NotSupportedException(); }
		}

		// Unsupported Methods.
		void IBindingList.AddIndex(PropertyDescriptor property) 
		{
			throw new NotSupportedException(); 
		}

		void IBindingList.ApplySort(PropertyDescriptor property, ListSortDirection direction) 
		{
			throw new NotSupportedException(); 
		}

		int IBindingList.Find(PropertyDescriptor property, object key) 
		{
			throw new NotSupportedException(); 
		}

		void IBindingList.RemoveIndex(PropertyDescriptor property) 
		{
			throw new NotSupportedException(); 
		}

		void IBindingList.RemoveSort() 
		{
			throw new NotSupportedException(); 
		}

		#endregion

		#region IListSource Members
		/// <summary>
		/// Return the container; this is important because the binder uses this to discover the supported interfaces
		/// </summary>
		/// <returns>the list of items</returns>
		public IList GetList()
		{
			return this;
		}

		/// <summary>
		/// Is this a list of lists?  No.  
		/// </summary>
		public bool ContainsListCollection { get { return true; } }
		#endregion
	}

	public class AccountCollectionEnumerator : IEnumerator 
	{
		int _cursor = -1, _next = -1;
		AccountCollection _coll;

		public AccountCollectionEnumerator(AccountCollection ParentCollectionCollection)
		{
			_coll = ParentCollectionCollection;
			_next = _coll.Count > 0 ? 0 : -1;
		}

		#region IEnumerator Members

		public virtual void Reset()
		{
			_cursor = -1;
			_next = _coll.Count > 0 ? 0 : -1;
		}

		object IEnumerator.Current
		{
			get
			{
				return Current;
			}
		}

		public Account Current
		{
			get
			{
				return _coll[_cursor];
			}
		}

		public bool MoveNext()
		{
			if (_next == -1)
				return false;

			_cursor = _next;

			_next++;
			if (_next >= _coll.Count)
				_next = -1;

			return _cursor != -1;
		}
		#endregion

	}


}
