using System;
using System.Collections;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Globalization;
using System.Text;
using System.Threading;
using System.Windows.Forms;

using Syncfusion.ComponentModel;
using Syncfusion.Drawing;
using Syncfusion.Styles;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;


namespace AccountTest
{
	[TypeConverter(typeof(ExpandableObjectConverter))]
	public class AccountPnL
	{
		public string Account;
		public double Amount;
		public bool Include;
		
		private AccountPnLCollection _parentCollection;
		private AccountPnLCollection _children = new AccountPnLCollection();
        
		#region ctor

		/// <summary>
		/// Default ctor needed for Xml and CodeDom serialization of strong-typed collections and
		/// for Essential Grid to be able to detect properties of items in strong-typed collections.
		/// </summary>
		public AccountPnL(string accountID)
		{
			Account = accountID;
			Amount = 0.0;
			Include = true;
		}

		#endregion 

		[Browsable(true)]
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Content)]
		public AccountPnLCollection Children 
		{
			get 
			{
				return _children;
			}
			set 
			{
				if (!Object.ReferenceEquals(Children, value))
				{
					_children = value;
				}
			}
		}

		public bool ShouldSerializeChildren()
		{
			return _children != null && _children.Count > 0;
		}

		[Browsable(false)] // do not show as field in grid
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
		public AccountPnLCollection ParentCollection 
		{
			get { return _parentCollection;	}
			set { _parentCollection = value; }
		}

		public bool ShouldSerializeParentCollection()
		{
			return false;
		}


	}


	public class AccountPnLCollection : IList, IBindingList
	{
		ArrayList inner = new ArrayList();

		public AccountPnLCollection()
		{
		}

		public override string ToString()
		{
			return GetType().Name.ToString() + "{ " + String.Format("Count {0}", Count) + " }";
		}

		public AccountPnL this[int index]
		{
			get
			{
				return (AccountPnL) inner[index];
			}
			set
			{
				if (!Object.ReferenceEquals(inner[index], value))
				{
					inner[index] = value;
					value.ParentCollection = this;
					RaiseListChanged(new ListChangedEventArgs(ListChangedType.ItemChanged, index, index));
				}
			}
		}

		public AccountPnL this[string account]
		{
			get 
			{
				foreach(AccountPnL acct in inner)
					if(acct.Account == account)
						return acct;

				return null;
			}
		}

		public bool Contains(AccountPnL value)
		{
			if (value == null)
				return false;

			return inner.Contains(value);
		}

		public int IndexOf(AccountPnL value)
		{
			return inner.IndexOf(value);
		}

		public void CopyTo(AccountPnL[] array, int index)
		{
			int count = Count;
			for (int n = 0; n < count; n++)
				array[index+n] = this[n];
		}

		public AccountPnLCollection SyncRoot
		{
			get
			{
				throw new NotSupportedException();
			}
		}

		public AccountPnLCollectionEnumerator GetEnumerator()
		{
			return new AccountPnLCollectionEnumerator(this);
		}
		
		public void Insert(int index, AccountPnL value)
		{
			inner.Insert(index, value);
			value.ParentCollection = this;
			RaiseListChanged(new ListChangedEventArgs(ListChangedType.ItemAdded, index, -1));
		}
	
		public void Remove(AccountPnL value)
		{
			int index = IndexOf(value);
			RemoveAt(index);
		}
		
		public int Add(AccountPnL value)
		{
			int index = inner.Add(value);
			value.ParentCollection = this;
			RaiseListChanged(new ListChangedEventArgs(ListChangedType.ItemAdded, index, -1));
			return index;
		}

		public void RemoveAt(int index)
		{
			inner.RemoveAt(index);
			RaiseListChanged(new ListChangedEventArgs(ListChangedType.ItemDeleted, index, -1));
		}

		public void Clear()
		{
			if (inner.Count > 0)
			{
				inner.Clear();
				RaiseListChanged(new ListChangedEventArgs(ListChangedType.Reset, -1, -1));
			}
		}

		public bool IsReadOnly
		{
			get
			{
				return false;
			}
		}

		public bool IsFixedSize
		{
			get
			{
				return false;
			}
		}

		public bool IsSynchronized
		{
			get
			{
				return false;
			}
		}

		public int Count
		{
			get
			{
				return inner.Count;
			}
		}


		public void NotifyItemPropertyChanged(AccountPnL item, string propertyName)
		{
			this.RaiseListChanged(new ListChangedEventArgs(ListChangedType.ItemChanged, IndexOf(item), -1));
		}

		#region IList Private Members

		object IList.this[int index]
		{
			get
			{
				return this[index];
			}
			set
			{
				this[index] = (AccountPnL) value;
			}
		}

		void IList.Insert(int index, object value)
		{
			Insert(index, (AccountPnL) value);
		}

		void IList.Remove(object value)
		{
			Remove((AccountPnL) value);
		}

		bool IList.Contains(object value)
		{
			return Contains((AccountPnL) value);
		}

		int IList.IndexOf(object value)
		{
			return IndexOf((AccountPnL) value);
		}

		int IList.Add(object value)
		{
			return Add((AccountPnL) value);
		}

		#endregion

		#region ICollection Private Members

		void ICollection.CopyTo(Array array, int index)
		{
			CopyTo((AccountPnL[]) array, index);
		}

		object ICollection.SyncRoot
		{
			get
			{
				return null;
			}
		}

		#endregion

		#region IEnumerable Private Members

		IEnumerator IEnumerable.GetEnumerator()
		{
			return GetEnumerator();
		}

		#endregion

		#region IBindingList Members

		// Implements IBindingList.
		public bool AllowEdit 
		{ 
			get { return true ; }
		}

		public bool AllowNew 
		{ 
			get { return true ; }
		}

		public bool AllowRemove 
		{ 
			get { return true ; }
		}

		public bool SupportsChangeNotification 
		{ 
			get { return true ; }
		}
        
		public bool SupportsSearching 
		{ 
			get { return false ; }
		}

		public bool SupportsSorting 
		{ 
			get { return false ; }
		}


		// Events.
		
		public event ListChangedEventHandler ListChanged;
		
		protected virtual void OnListChanged(ListChangedEventArgs e)
		{
			if (ListChanged != null)
				ListChanged(this, e);
		}
		
		public void RaiseListChanged(ListChangedEventArgs e)
		{
			OnListChanged(e);
		}
		
		// Methods.

		public AccountPnL AddNew()
		{
			AccountPnL c = new AccountPnL(this.Count.ToString());
			new AddNewObjectListener(this, c, Add(c));
			return c;
		}

		object IBindingList.AddNew() 
		{
			return AddNew();
		}

		public class AddNewObjectListener : IEditableObject	
		{
			AccountPnLCollection AccountPnLs;
			AccountPnL item;
			int index;

			public AddNewObjectListener(AccountPnLCollection AccountPnLs, AccountPnL item, int index)
			{
				this.AccountPnLs = AccountPnLs;
				this.item = item;
				this.index = index;
				//item.ForwardEditableObject = this;
			}

			public void Detach()
			{
				//item.ForwardEditableObject = null;
				AccountPnLs = null;
				item = null;
			}

			#region IEditableObject Members

			public void EndEdit()
			{
				//AccountPnLs.RaiseListChanged(new ListChangedEventArgs(ListChangedType.ItemChanged, index, index));

				Detach();
			}

			public void CancelEdit()
			{
				// When a AddNew operation is canceld by the user the object needs to be removed again from
				// the list.
				AccountPnLs.RemoveAt(index);

				Detach();
			}

			public void BeginEdit()
			{
			}

			#endregion
		}

		// Unsupported properties.
		bool IBindingList.IsSorted 
		{ 
			get { throw new NotSupportedException(); }
		}

		ListSortDirection IBindingList.SortDirection 
		{ 
			get { throw new NotSupportedException(); }
		}


		PropertyDescriptor IBindingList.SortProperty 
		{ 
			get { throw new NotSupportedException(); }
		}

		// Unsupported Methods.
		void IBindingList.AddIndex(PropertyDescriptor property) 
		{
			throw new NotSupportedException(); 
		}

		void IBindingList.ApplySort(PropertyDescriptor property, ListSortDirection direction) 
		{
			throw new NotSupportedException(); 
		}

		int IBindingList.Find(PropertyDescriptor property, object key) 
		{
			throw new NotSupportedException(); 
		}

		void IBindingList.RemoveIndex(PropertyDescriptor property) 
		{
			throw new NotSupportedException(); 
		}

		void IBindingList.RemoveSort() 
		{
			throw new NotSupportedException(); 
		}

		#endregion

	}

	public class AccountPnLCollectionEnumerator : IEnumerator 
	{
		int _cursor = -1, _next = -1;
		AccountPnLCollection _coll;

		public AccountPnLCollectionEnumerator(AccountPnLCollection ParentCollectionCollection)
		{
			_coll = ParentCollectionCollection;
			_next = _coll.Count > 0 ? 0 : -1;
		}

		#region IEnumerator Members

		public virtual void Reset()
		{
			_cursor = -1;
			_next = _coll.Count > 0 ? 0 : -1;
		}

		object IEnumerator.Current
		{
			get
			{
				return Current;
			}
		}

		public AccountPnL Current
		{
			get
			{
				return _coll[_cursor];
			}
		}

		public bool MoveNext()
		{
			if (_next == -1)
				return false;

			_cursor = _next;

			_next++;
			if (_next >= _coll.Count)
				_next = -1;

			return _cursor != -1;
		}
		#endregion

	}


}
