using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms;

namespace AccountTest
{
	/// <summary>
	/// Summary description for AccountTest.
	/// </summary>
	public class AcctTest : System.Windows.Forms.Form
	{
		private Syncfusion.Windows.Forms.Grid.GridDataBoundGrid _grid;
		private System.Windows.Forms.Timer _timer;
		private System.ComponentModel.IContainer components;
		//private Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl _gridGp;

		// Custom Collection to be used as DataSource for the grid
		private AccountCollection _accounts;
		
		public AcctTest()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			_grid.Binder.DirectSaveCellInfo = true;
			_grid.Binder.OptimizeListChangedEvent = true;
			_grid.UseListChangedEvent = true;
			_grid.ForceUpdateAfterListChangedEvent = true;
			_grid.Binder.ForceUpdateAfterListChangedEvent = true;
			
			_grid.Model.Options.SelectCellsMouseButtonsMask = MouseButtons.Left;
			_grid.Model.ColsMoving += new GridRangeMovingEventHandler(Model_ColsMoving);

			// Hydrate the _accounts collection and setup the hierarchy
			InitializeDataSource();
			InitializeGridHierarchy();

			// Enable the timer
			_timer.Tick += new EventHandler(_timer_Tick);
			_timer.Enabled = true;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this._grid = new Syncfusion.Windows.Forms.Grid.GridDataBoundGrid();
			this._timer = new System.Windows.Forms.Timer(this.components);
			((System.ComponentModel.ISupportInitialize)(this._grid)).BeginInit();
			this.SuspendLayout();
			// 
			// _grid
			// 
			this._grid.AllowDragSelectedCols = true;
			this._grid.Dock = System.Windows.Forms.DockStyle.Fill;
			this._grid.Location = new System.Drawing.Point(0, 0);
			this._grid.Name = "_grid";
			this._grid.OptimizeInsertRemoveCells = true;
			this._grid.ShowCurrentCellBorderBehavior = Syncfusion.Windows.Forms.Grid.GridShowCurrentCellBorder.GrayWhenLostFocus;
			this._grid.Size = new System.Drawing.Size(424, 326);
			this._grid.SmartSizeBox = false;
			this._grid.SortBehavior = Syncfusion.Windows.Forms.Grid.GridSortBehavior.DoubleClick;
			this._grid.TabIndex = 0;
			this._grid.Text = "gridDataBoundGrid1";
			// 
			// _timer
			// 
			this._timer.Interval = 1000;
			// 
			// AcctTest
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(424, 326);
			this.Controls.Add(this._grid);
			this.Name = "AcctTest";
			this.Text = "AccountTest";
			this.Load += new System.EventHandler(this.AcctTest_Load);
			((System.ComponentModel.ISupportInitialize)(this._grid)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

//		[STAThread]
//		static void Main()
//		{
//			AcctTest frm = new AcctTest();
//			Application.Run(frm);
//		}

		private void InitializeDataSource()
		{
			// Get a dictionary of Account - SubAccount mappings
			IDictionary accts = _getAccountMap();
			
			// Create the AccountCollection using the information in the accts map
			_accounts = new AccountCollection();
			foreach (DictionaryEntry entry in accts)
			{
				string acctId = (string) entry.Key;
				string[] subAccts = (string[]) entry.Value;

				// Create new Account object and add the Sub Accounts as Children of the Account
				Account account = new Account(acctId);
				foreach(string subAcctId in subAccts)
				{
					Account subAccount = new Account(subAcctId);
					account.Children.Add(subAccount);
				}

				_accounts.Add(account);
			}
		}

		private void InitializeGridHierarchy()
		{
			string[] cols = new string[2];
			cols[0] = "AccountId";
			cols[1] = "Total";

			_grid.Binder.ResetHierarchyLevels();
			_grid.Binder.SetDataBinding(_accounts, "");

			GridHierarchyLevel hlAccount = _grid.Binder.RootHierarchyLevel;
			hlAccount.LayoutColumns(cols);
			hlAccount.ShowHeaders = true;

			// Add the same columns for the SubAccounts
			for (int i = 1; i < hlAccount.GridBoundColumns.Count; i++)
			{
				GridBoundColumn gbColumn = hlAccount.GridBoundColumns[i];
				gbColumn.HeaderText = cols[i-1];
			}
			
			GridHierarchyLevel hlSubAccount = _grid.Binder.AddRelation("Children");
			hlSubAccount.LayoutColumns(cols);
			hlSubAccount.ShowHeaders = false;
			hlSubAccount.RowStyle.BackColor = Color.FromArgb(0xEE, 0xEE, 0xEE);
		}

		private IDictionary _getAccountMap()
		{
			IDictionary map = new Hashtable();
			map["AA"] = new string[]{"aa001", "aa002"};
			map["BB"] = new string[]{"bb001", "bb002"};
			map["CC"] = new string[]{"cc001", "cc002"};
			map["DD"] = new string[]{"dd001", "dd002"};
			map["EE"] = new string[]{"ee001", "ee002"};
			return map;
		}

		private void UpdateDataSource()
		{
			_grid.BeginUpdate(BeginUpdateOptions.Invalidate);
			foreach (Account acct in _accounts)
			{
				foreach(Account subAcct in acct.Children)
				{
					if(subAcct.Total < 1000)
					{
						subAcct.Total++;
						acct.Total++;
					}
				}
			}
			_grid.Refresh();
			_grid.EndUpdate(true);
		}

		private void Model_ColsMoving(object sender, GridRangeMovingEventArgs e)
		{
			// do not allow columns to move
			e.Cancel = true;
		}

		private void _timer_Tick(object sender, EventArgs e)
		{
			_timer.Enabled = false;
			UpdateDataSource();
			_timer.Enabled = true;
		}

		private void AcctTest_Load(object sender, System.EventArgs e)
		{
			_grid.ExpandAll();
		}
	}
}
