using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;

namespace AccountTest
{
	/// <summary>
	/// Summary description for AccountTest.
	/// </summary>
	public class AcctTestNew : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Timer _timer;
		private System.ComponentModel.IContainer components;
		private Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl _gridGpCtrl;

		// Custom Collection to be used as DataSource for the grid
		private AccountCollection _accounts;
		
		public AcctTestNew()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// Hydrate the _accounts collection and setup the hierarchy
			InitializeDataSource();
			InitializeGridHierarchy();

			// Enable the timer
			_timer.Tick += new EventHandler(_timer_Tick);
			_timer.Enabled = true;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this._timer = new System.Windows.Forms.Timer(this.components);
			this._gridGpCtrl = new Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl();
			((System.ComponentModel.ISupportInitialize)(this._gridGpCtrl)).BeginInit();
			this.SuspendLayout();
			// 
			// _timer
			// 
			this._timer.Interval = 1000;
			// 
			// _gridGpCtrl
			// 
			this._gridGpCtrl.BackColor = System.Drawing.SystemColors.Window;
			this._gridGpCtrl.ChildGroupOptions.ShowAddNewRecordBeforeDetails = false;
			this._gridGpCtrl.ChildGroupOptions.ShowCaption = false;
			this._gridGpCtrl.ChildGroupOptions.ShowSummaries = false;
			this._gridGpCtrl.Dock = System.Windows.Forms.DockStyle.Fill;
			this._gridGpCtrl.Location = new System.Drawing.Point(0, 0);
			this._gridGpCtrl.Name = "_gridGpCtrl";
			this._gridGpCtrl.NestedTableGroupOptions.ShowAddNewRecordAfterDetails = false;
			this._gridGpCtrl.NestedTableGroupOptions.ShowAddNewRecordBeforeDetails = false;
			this._gridGpCtrl.NestedTableGroupOptions.ShowCaption = false;
			this._gridGpCtrl.NestedTableGroupOptions.ShowCaptionPlusMinus = true;
			this._gridGpCtrl.NestedTableGroupOptions.ShowColumnHeaders = true;
			this._gridGpCtrl.NestedTableGroupOptions.ShowSummaries = false;
			this._gridGpCtrl.Size = new System.Drawing.Size(496, 350);
			this._gridGpCtrl.TabIndex = 0;
			this._gridGpCtrl.TableDescriptor.AllowEdit = false;
			this._gridGpCtrl.TableDescriptor.AllowNew = false;
			this._gridGpCtrl.TableDescriptor.AllowRemove = false;
			this._gridGpCtrl.TableDescriptor.ChildGroupOptions.ShowCaption = false;
			this._gridGpCtrl.TableDescriptor.ChildGroupOptions.ShowSummaries = false;
			this._gridGpCtrl.TableDescriptor.Name = "Default";
			this._gridGpCtrl.TableOptions.AllowDragColumns = false;
			this._gridGpCtrl.TableOptions.AllowSortColumns = false;
			this._gridGpCtrl.TableOptions.CaptionRowHeight = 22;
			this._gridGpCtrl.TableOptions.ShowRowHeader = false;
			this._gridGpCtrl.TopLevelGroupOptions.ShowAddNewRecordAfterDetails = true;
			this._gridGpCtrl.TopLevelGroupOptions.ShowAddNewRecordBeforeDetails = false;
			this._gridGpCtrl.TopLevelGroupOptions.ShowCaption = false;
			this._gridGpCtrl.TopLevelGroupOptions.ShowSummaries = false;
			// 
			// AcctTestNew
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(496, 350);
			this.Controls.Add(this._gridGpCtrl);
			this.Name = "AcctTestNew";
			this.Text = "AccountTestNew";
			((System.ComponentModel.ISupportInitialize)(this._gridGpCtrl)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		[STAThread]
		static void Main()
		{
			AcctTestNew frm = new AcctTestNew();
			Application.Run(frm);
		}

		private void InitializeDataSource()
		{
			// Get a dictionary of Account - SubAccount mappings
			IDictionary accts = _getAccountMap();
			
			// Create the AccountCollection using the information in the accts map
			_accounts = new AccountCollection();
			foreach (DictionaryEntry entry in accts)
			{
				string acctId = (string) entry.Key;
				string[] subAccts = (string[]) entry.Value;

				// Create new Account object and add the Sub Accounts as Children of the Account
				Account account = new Account(acctId);
				foreach(string subAcctId in subAccts)
				{
					Account subAccount = new Account(subAcctId);
					account.Children.Add(subAccount);
				}

				_accounts.Add(account);
			}
		}

		private void InitializeGridHierarchy()
		{
			this._gridGpCtrl.TableControl.HorizontalScrollTips = false;
			this._gridGpCtrl.BorderStyle = BorderStyle.FixedSingle;
			this._gridGpCtrl.FilterRuntimeProperties = true;

			this._gridGpCtrl.DataSource = _accounts;

			GridEngine engine = this._gridGpCtrl.Engine;
			engine.TopLevelGroupOptions.ShowCaption = true;

			GridRelationDescriptorCollection relations = new GridRelationDescriptorCollection();

			// First level
			GridRelationDescriptor rd = AddRelation("Level_0", relations);
			this._gridGpCtrl.TableDescriptor.Relations = relations;

			this._gridGpCtrl.TableOptions.ShowRecordPlusMinus = true;
			this._gridGpCtrl.NestedTableGroupOptions.ShowCaptionPlusMinus = false;
			this._gridGpCtrl.Table.ExpandAllRecords();
			Syncfusion.Grouping.Diagnostics.IterateThroughNestedDisplayElement(this._gridGpCtrl.Table);
		}

		private GridRelationDescriptor AddRelation(string name, GridRelationDescriptorCollection relations)
		{
			GridRelationDescriptor children = new GridRelationDescriptor();
			children.RelationKind = RelationKind.UniformChildList;
			children.MappingName = "Children";
			children.Name = name;
			relations.Add(children);
			return children;
		}

		private IDictionary _getAccountMap()
		{
			IDictionary map = new Hashtable();
			map["AA"] = new string[]{"aa001", "aa002"};
			map["BB"] = new string[]{"bb001", "bb002"};
			map["CC"] = new string[]{"cc001", "cc002"};
			map["DD"] = new string[]{"dd001", "dd002"};
			map["EE"] = new string[]{"ee001", "ee002"};
			return map;
		}

		private void UpdateDataSource()
		{
			//_gridGpCtrl.TableControl.BeginUpdate(BeginUpdateOptions.Invalidate);
			foreach (Account acct in _accounts)
			{
				foreach(Account subAcct in acct.Children)
				{
					if(subAcct.Total < 1000)
					{
						subAcct.Total++;
						acct.Total++;
					}
				}
			}

			//_gridGpCtrl.TableControl.EndUpdate();
		}

		private void _timer_Tick(object sender, EventArgs e)
		{
			_timer.Enabled = false;
			UpdateDataSource();
			_timer.Enabled = true;
		}
	}
}
