using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using AccountTest;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms;

namespace AccountTest
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		#region Entry Points
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Form1 frm1 = new Form1();
			Application.Run(frm1);
		}
		#endregion

		#region Instance fields and form elements
		private System.ComponentModel.IContainer components;
		private Syncfusion.Windows.Forms.Grid.GridDataBoundGrid _grid;
		private System.Windows.Forms.Timer _timer;

		private AccountPnLCollection _accounts;
		#endregion
		
		#region Construction and initialization
		public Form1()
		{
			CreateDataSource();
			InitializeComponent();

			_grid.Binder.DirectSaveCellInfo = true;
			_grid.Binder.OptimizeListChangedEvent = true;
			
			_grid.Model.Options.SelectCellsMouseButtonsMask = MouseButtons.Left;
			_grid.Model.QueryCellFormattedText += new GridCellTextEventHandler(QueryCellFormattedText);
			_grid.Model.QueryCellInfo +=new GridQueryCellInfoEventHandler(QueryCellInfo);
			_grid.Model.ColsMoving += new GridRangeMovingEventHandler(Model_ColsMoving);

			InitializeGridHierarchy();
		}
		#endregion

		#region Data source initialization
		private void CreateDataSource()
		{
			ArrayList excludeGroups = new ArrayList(new string[]{"22", "44"}); 
			_accounts = new AccountPnLCollection();
			for (int i=0; i<5; i++)
			{
				string traderID = "Trader" + i.ToString();
				string[] accountGroups = new string[]{String.Format("{0}{1}", i, i), String.Format("{0}{1}{2}", i, i, i)};

				AccountPnL acct = new AccountPnL(traderID);
				foreach(string accountGroup in accountGroups)
				{
					AccountPnL acctChild = new AccountPnL(accountGroup);
					acctChild.Include = !excludeGroups.Contains(accountGroup);
					acct.Children.Add(acctChild);
				}

				_accounts.Add(acct);
			}

			AccountPnL acctTot = new AccountPnL("TOTAL");
			acctTot.Include = true;
			_accounts.Add(acctTot);
		}

		private void InitializeGridHierarchy()
		{
			ArrayList headerTextList = new ArrayList(new string[]{"Amount"});
			ArrayList mappingNameList = new ArrayList(new string[]{"Amount"});
			ArrayList widthList = new ArrayList(new int[]{50});

			string[] cols = new string[mappingNameList.Count + 2];
			cols[0] = "Include";
			cols[1] = "Account";
			mappingNameList.CopyTo(cols, 2);

			_grid.Binder.ResetHierarchyLevels();
			_grid.Binder.SetDataBinding(_accounts, "");

			GridHierarchyLevel hlTrader = _grid.Binder.RootHierarchyLevel;
			hlTrader.LayoutColumns(cols);
			hlTrader.ShowHeaders = true;

			for (int i = 2; i < hlTrader.GridBoundColumns.Count; i++)
			{
				GridBoundColumn gbColumn = hlTrader.GridBoundColumns[i];
				gbColumn.HeaderText = (string) headerTextList[i - 2];
			}
			
			GridHierarchyLevel hlTraderAcctGp = _grid.Binder.AddRelation("Children");
			hlTraderAcctGp.LayoutColumns(cols);
			hlTraderAcctGp.ShowHeaders = false;
			hlTraderAcctGp.RowStyle.BackColor = Color.FromArgb(0xEE, 0xEE, 0xEE);
			
			_setStyles(hlTraderAcctGp);
			
			GridModelRowColOperations gridModelCols = _grid.Model.Cols;
			gridModelCols.FrozenCount = 3;
			gridModelCols.Size.SetRange(gridModelCols.HeaderCount + 3,
				widthList.Count + (gridModelCols.HeaderCount + 3) - 1,
				(int[]) widthList.ToArray(typeof(int)));
		}
		#endregion
		
		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			Syncfusion.Windows.Forms.Grid.GridBaseStyle gridBaseStyle1 = new Syncfusion.Windows.Forms.Grid.GridBaseStyle();
			Syncfusion.Windows.Forms.Grid.GridBaseStyle gridBaseStyle2 = new Syncfusion.Windows.Forms.Grid.GridBaseStyle();
			Syncfusion.Windows.Forms.Grid.GridBaseStyle gridBaseStyle3 = new Syncfusion.Windows.Forms.Grid.GridBaseStyle();
			Syncfusion.Windows.Forms.Grid.GridBaseStyle gridBaseStyle4 = new Syncfusion.Windows.Forms.Grid.GridBaseStyle();
			Syncfusion.Windows.Forms.Grid.GridBaseStyle gridBaseStyle5 = new Syncfusion.Windows.Forms.Grid.GridBaseStyle();
			Syncfusion.Windows.Forms.Grid.GridBaseStyle gridBaseStyle6 = new Syncfusion.Windows.Forms.Grid.GridBaseStyle();
			Syncfusion.Windows.Forms.Grid.GridStyleInfo gridStyleInfo1 = new Syncfusion.Windows.Forms.Grid.GridStyleInfo();
			this._grid = new Syncfusion.Windows.Forms.Grid.GridDataBoundGrid();
			this._timer = new System.Windows.Forms.Timer(this.components);
			((System.ComponentModel.ISupportInitialize)(this._grid)).BeginInit();
			// 
			// _grid
			// 
			this._grid.AllowDragSelectedCols = true;
			this._grid.AllowResizeToFit = false;
			this._grid.BackColor = System.Drawing.Color.White;
			gridBaseStyle1.Name = "Header";
			gridBaseStyle1.StyleInfo.Borders.Bottom = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.None);
			gridBaseStyle1.StyleInfo.Borders.Left = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.None);
			gridBaseStyle1.StyleInfo.Borders.Right = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.None);
			gridBaseStyle1.StyleInfo.Borders.Top = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.None);
			gridBaseStyle1.StyleInfo.CellType = "Header";
			gridBaseStyle1.StyleInfo.Font.Bold = true;
			gridBaseStyle1.StyleInfo.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.Color.White);
			gridBaseStyle1.StyleInfo.TextColor = System.Drawing.SystemColors.WindowText;
			gridBaseStyle1.StyleInfo.VerticalAlignment = Syncfusion.Windows.Forms.Grid.GridVerticalAlignment.Middle;
			gridBaseStyle2.Name = "Sub Header";
			gridBaseStyle2.StyleInfo.Font.Facename = "Verdana";
			gridBaseStyle2.StyleInfo.Font.Italic = true;
			gridBaseStyle3.Name = "Column Header";
			gridBaseStyle3.StyleInfo.BaseStyle = "Header";
			gridBaseStyle3.StyleInfo.CellType = "ColumnHeaderCell";
			gridBaseStyle3.StyleInfo.Enabled = false;
			gridBaseStyle3.StyleInfo.Font.Bold = true;
			gridBaseStyle3.StyleInfo.Font.Facename = "Verdana";
			gridBaseStyle3.StyleInfo.Font.Italic = true;
			gridBaseStyle3.StyleInfo.HorizontalAlignment = Syncfusion.Windows.Forms.Grid.GridHorizontalAlignment.Center;
			gridBaseStyle3.StyleInfo.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.Color.FromArgb(((System.Byte)(51)), ((System.Byte)(153)), ((System.Byte)(204))));
			gridBaseStyle3.StyleInfo.TextColor = System.Drawing.Color.White;
			gridBaseStyle4.Name = "Row Header";
			gridBaseStyle4.StyleInfo.BaseStyle = "Header";
			gridBaseStyle4.StyleInfo.CellType = "RowHeaderCell";
			gridBaseStyle4.StyleInfo.Enabled = true;
			gridBaseStyle4.StyleInfo.Font.Facename = "Verdana";
			gridBaseStyle4.StyleInfo.HorizontalAlignment = Syncfusion.Windows.Forms.Grid.GridHorizontalAlignment.Left;
			gridBaseStyle5.Name = "Footer";
			gridBaseStyle5.StyleInfo.Font.Bold = true;
			gridBaseStyle5.StyleInfo.Font.Italic = true;
			gridBaseStyle6.Name = "Standard";
			gridBaseStyle6.StyleInfo.Borders.Bottom = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.None);
			gridBaseStyle6.StyleInfo.Borders.Left = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.None);
			gridBaseStyle6.StyleInfo.Borders.Right = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.None);
			gridBaseStyle6.StyleInfo.Borders.Top = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.None);
			gridBaseStyle6.StyleInfo.CellType = "Static";
			gridBaseStyle6.StyleInfo.CheckBoxOptions.CheckedValue = "True";
			gridBaseStyle6.StyleInfo.CheckBoxOptions.IndetermValue = "Null";
			gridBaseStyle6.StyleInfo.CheckBoxOptions.UncheckedValue = "False";
			gridBaseStyle6.StyleInfo.Font.Facename = "Verdana";
			gridBaseStyle6.StyleInfo.HorizontalAlignment = Syncfusion.Windows.Forms.Grid.GridHorizontalAlignment.Right;
			gridBaseStyle6.StyleInfo.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.Color.White);
			gridBaseStyle6.StyleInfo.ReadOnly = false;
			gridBaseStyle6.StyleInfo.WrapText = false;
			this._grid.BaseStylesMap.AddRange(new Syncfusion.Windows.Forms.Grid.GridBaseStyle[] {
																									gridBaseStyle1,
																									gridBaseStyle2,
																									gridBaseStyle3,
																									gridBaseStyle4,
																									gridBaseStyle5,
																									gridBaseStyle6});
			this._grid.Dock = System.Windows.Forms.DockStyle.Fill;
			this._grid.EnableAddNew = false;
			this._grid.EnableRemove = false;
			this._grid.ExcelLikeCurrentCell = true;
			this._grid.Font = new System.Drawing.Font("Verdana", 8.25F);
			this._grid.ForceUpdateAfterListChangedEvent = false;
			this._grid.HScrollPixel = true;
			this._grid.Location = new System.Drawing.Point(0, 0);
			this._grid.Name = "_grid";
			this._grid.OptimizeInsertRemoveCells = true;
			this._grid.Properties.BackgroundColor = System.Drawing.Color.White;
			this._grid.Properties.DisplayHorzLines = false;
			this._grid.Properties.DisplayVertLines = false;
			this._grid.Properties.MarkRowHeader = false;
			this._grid.Properties.PrintRowHeader = false;
			this._grid.Properties.RowHeaders = false;
			this._grid.ShowCurrentCellBorderBehavior = Syncfusion.Windows.Forms.Grid.GridShowCurrentCellBorder.GrayWhenLostFocus;
			this._grid.SmartSizeBox = false;
			this._grid.SortBehavior = Syncfusion.Windows.Forms.Grid.GridSortBehavior.None;
			gridStyleInfo1.Font.Bold = false;
			gridStyleInfo1.Font.Facename = "Verdana";
			gridStyleInfo1.Font.Italic = false;
			gridStyleInfo1.Font.Size = 8.25F;
			gridStyleInfo1.Font.Strikeout = false;
			gridStyleInfo1.Font.Underline = false;
			this._grid.TableStyle = gridStyleInfo1;
			this._grid.UseListChangedEvent = true;
			// 
			// _timer
			// 
			this._timer.Interval = 1000;
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.Color.White;
			this.ClientSize = new System.Drawing.Size(520, 318);
			this.Name = "Form1";
			this.StartPosition = System.Windows.Forms.FormStartPosition.Manual;
			this.Text = "Desk P&L";
			this.Load += new System.EventHandler(this.Form1_Load);
			((System.ComponentModel.ISupportInitialize)(this._grid)).EndInit();

		}
		#endregion

		#region Styles and Formatting
		private void _setStyles(GridHierarchyLevel hl)
		{
			_grid.Binder.GridBoundColumns["Include"].StyleInfo.TriState = true;
			_grid.Binder.GridBoundColumns["Account"].StyleInfo.HorizontalAlignment = GridHorizontalAlignment.Left;
			_grid.Model.Cols.Size.SetRange(_grid.Model.Cols.HeaderCount + 1, _grid.Model.Cols.HeaderCount + 1, 20);
		}
		#endregion

		#region GUI Events
		private void Form1_Load(object sender, System.EventArgs e)
		{
			_timer.Tick += new EventHandler(_timer_Tick);
			_timer.Enabled = true;
		}

		private void QueryCellFormattedText(object sender, GridCellTextEventArgs e)
		{
			if(e.Value != DBNull.Value)
			{
				int col = this._grid.Binder.ColIndexToField(e.Style.CellIdentity.ColIndex);
				int row = this._grid.Binder.RowIndexToListManagerPosition(e.Style.CellIdentity.RowIndex);
				if ((row >= 0) && (col >= 0))
				{
					if( col < _grid.Binder.GridBoundColumns.Count ) 
					{
						GridBoundColumn gbColumn = _grid.Binder.GridBoundColumns[col];
						if( gbColumn.HeaderText == "Account" )
							gbColumn.StyleInfo.HorizontalAlignment = e.Style.HorizontalAlignment = GridHorizontalAlignment.Left;
					}
				}
			}
		}

		private void QueryCellInfo(object sender, GridQueryCellInfoEventArgs e)
		{
			try
			{
				if (e.RowIndex == -1)
				{
					if (e.ColIndex == 3)
						e.Style.Font.Italic = true;
				}
				
				if (e.ColIndex == -1)
				{
					if ((e.RowIndex == _grid.Model.RowCount) && (e.ColIndex == -1))
					{
						e.Style.Font.Bold = true;
						e.Style.Borders.Top = new GridBorder(GridBorderStyle.Solid, Color.Black, GridBorderWeight.Thin);
						e.Style.Borders.Bottom = new GridBorder(GridBorderStyle.Solid, Color.Black, GridBorderWeight.Thick);
					}
					else
					{
						GridBoundRecordState recordState =  _grid.Binder.GetRecordStateAtRowIndex(e.RowIndex);
						if ((recordState != null) &&(recordState.LevelIndex == 1))
							e.Style.BackColor = Color.FromArgb(0xEE, 0xEE, 0xEE);
					}
				}
				else 
				{
					int col = this._grid.Binder.ColIndexToField(e.ColIndex);
					if (col >= 0 && col < _grid.Binder.GridBoundColumns.Count )
					{
						GridBoundColumn gbColumn = _grid.Binder.GridBoundColumns[col];
						if( gbColumn.HeaderText == "Account" )
							gbColumn.StyleInfo.HorizontalAlignment = e.Style.HorizontalAlignment = GridHorizontalAlignment.Left;
					}
				}
			}
			catch (Exception ex)
			{
				Console.WriteLine(ex.Message);
			}
		}

		private void Model_ColsMoving(object sender, GridRangeMovingEventArgs e)
		{
			// do not allow columns to move
			e.Cancel = true;
		}
		#endregion

		#region IDisposable implementation details
		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (components != null) components.Dispose();
			}
			base.Dispose(disposing);
		}
		#endregion

		private void _timer_Tick(object sender, EventArgs e)
		{
			foreach(AccountPnL acct in _accounts)
			{
				double amt = 0.0;
				foreach(AccountPnL childAcct in acct.Children)
				{
					if(childAcct.Amount < 100)
						childAcct.Amount++;
					else
						childAcct.Amount = 0.0;

					amt += childAcct.Amount;
				}

				acct.Amount = amt;
			}
		}
	}
}
